#ifndef FrameCPP_VERSION_3_FrHistory_HH
#define FrameCPP_VERSION_3_FrHistory_HH

#include "framecpp/FrameCPP.hh"

#include "framecpp/Version3/FrameSpec.hh"
#include "framecpp/Version3/STRING.hh"

namespace FrameCPP
{
  namespace Version_3
  {
    class FrHistory
      : public Common::FrameSpec::Object
    {
    public:
      //: Default constructor
      FrHistory( );

      FrHistory( const FrHistory& Source );

      FrHistory( const std::string& Name,
		 INT_4U Time,
		 const std::string& Comment );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrHistory* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      virtual std::string ErrorInfo( ) const;

      // Accessors
      const std::string& GetName() const;

      INT_4U GetTime( ) const
      {
         return m_data.time;
      }   
      const std::string& GetComment( ) const
      {
 	 return m_data.comment;
      }   

      //: Merge with another FrAdcData
      FrHistory& Merge( const FrHistory& RHS );

      //-----------------------------------------------------------------
      /// \brief Promote object
      ///
      /// \param[in] Source
      /// \param[in] Obj
      /// \param[in] Stream
      ///
      /// \return
      ///     Promoted version of Obj.
      //-----------------------------------------------------------------
      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      /// \brief comparison operator
      bool operator==( const FrHistory& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      struct fr_history_data_type {
	STRING	name;
	INT_4U	time;
	STRING	comment;

	bool operator==( const fr_history_data_type& RHS ) const;
      };

      fr_history_data_type	m_data;

      FrHistory( Common::IStream& Stream );
    };

    inline const char* const FrHistory::
    StructName( )
    {
      static const char* const name = "FrHistory";
      return name;
    }

    inline bool FrHistory::
    operator==( const FrHistory& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }
  } // namespace - Version_3
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_3_FrHistory_HH */
