#ifndef GeneralRegexHH
#define GeneralRegexHH

#include <string>
#include <regex.h>
#include "ldasexception.hh"


//-----------------------------------------------------------------------------
/// \brief  POSIX Regular Expression
///
/// This class represents a POSIX regular expression.
///
class Regex
{
public:

    /* Constructors/Destructor */
    Regex( const std::string& regex, int flags = REG_EXTENDED )
        throw( LdasException, std::bad_alloc );
    ~Regex() throw();

    /* Accessors */
    const regex_t& getPattern() const throw();
    const std::string& getRegex() const throw();
    
private:

    /* Data */
    std::string mRegex;
    regex_t mPattern;
};


//-----------------------------------------------------------------------------
/// \brief  Get Compiled Pattern
///
/// \return const regex_t &
///
inline const regex_t& Regex::getPattern() const throw()
{
    return mPattern;
}


//-----------------------------------------------------------------------------
/// \brief  Get Regular Expression
///
/// \return const std::string&
///
inline const std::string& Regex::getRegex() const throw()
{
    return mRegex;
}


#endif // GeneralRegexHH
